import { GraphQLError } from 'graphql/error';
import { GraphQLObjectType, GraphQLFieldResolver } from 'graphql/type/definition';
import { GraphQLSchema } from 'graphql/type/schema';
import { DocumentNode, OperationDefinitionNode, FragmentDefinitionNode } from 'graphql/language/ast';
import { ExecutionResult } from 'graphql/execution/execute';
export declare type MaybePromise<T> = Promise<T> | T;
declare type FieldValue = Record<string, {}> | Record<string, {}>[] | string | number | boolean | null;
declare type PatchBundle = Promise<{
    patch: ExecutionPatchResult;
    dependentPatches: PatchBundle[];
}>;
export declare type ExecutionContext = {
    schema: GraphQLSchema;
    fragments: Record<string, FragmentDefinitionNode>;
    rootValue: {};
    contextValue: {};
    operation: OperationDefinitionNode;
    variableValues: {
        [variable: string]: {};
    };
    fieldResolver: GraphQLFieldResolver<any, any>;
    errors: Array<GraphQLError>;
    patchDispatcher?: PatchDispatcher;
    deferredDependents?: Record<string, {
        patches: PatchBundle[];
        errors: GraphQLError[];
    }>;
};
export interface ExecutionPatchResult {
    data?: FieldValue;
    errors?: ReadonlyArray<GraphQLError>;
    path: ReadonlyArray<string | number>;
}
export interface DeferredExecutionResult {
    initialResult: ExecutionResult;
    deferredPatches: AsyncIterable<ExecutionPatchResult>;
}
export declare function isDeferredExecutionResult(result: any): result is DeferredExecutionResult;
declare class PatchDispatcher {
    private resolvers;
    private resultPromises;
    dispatch(patch: PatchBundle): void;
    getAsyncIterable(): AsyncIterable<ExecutionPatchResult>;
}
export declare function execute(ExecutionArgs: any, ..._: any[]): MaybePromise<ExecutionResult | DeferredExecutionResult>;
export declare function execute(schema: GraphQLSchema, document: DocumentNode, rootValue?: {}, contextValue?: {}, variableValues?: {
    [variable: string]: {};
}, operationName?: string, fieldResolver?: GraphQLFieldResolver<any, any>): MaybePromise<ExecutionResult | DeferredExecutionResult>;
export declare function getOperationRootType(schema: GraphQLSchema, operation: OperationDefinitionNode): GraphQLObjectType;
export {};
//# sourceMappingURL=execute.d.ts.map